@@ -22,57 +22,19 @@ Currently there are still interdependencies that need to be cleaned.
 use 5.008;
 use strict;
 use warnings;
-use File::Spec  ();
 use Carp        ();
-use Wx qw{
-	wxSTC_LEX_ADA
-	wxSTC_LEX_ASM
-	wxSTC_LEX_BATCH
-	wxSTC_LEX_CPP
-	wxSTC_LEX_CSS
-	wxSTC_LEX_DIFF
-	wxSTC_LEX_EIFFEL
-	wxSTC_LEX_EIFFELKW
-	wxSTC_LEX_ERRORLIST
-	wxSTC_LEX_ESCRIPT
-	wxSTC_LEX_FORTRAN
-	wxSTC_LEX_FORTH
-	wxSTC_LEX_HTML
-	wxSTC_LEX_LATEX
-	wxSTC_LEX_LISP
-	wxSTC_LEX_LUA
-	wxSTC_LEX_MAKEFILE
-	wxSTC_LEX_MATLAB
-	wxSTC_LEX_PASCAL
-	wxSTC_LEX_PERL
-	wxSTC_LEX_PHPSCRIPT
-	wxSTC_LEX_PYTHON
-	wxSTC_LEX_RUBY
-	wxSTC_LEX_SQL
-	wxSTC_LEX_TCL
-	wxSTC_LEX_VBSCRIPT
-	wxSTC_LEX_YAML
-	wxSTC_LEX_XML
+use File::Spec  ();
+use Padre::Util ();
+use Padre::Wx   ();
 
-	wxSTC_LEX_AUTOMATIC
-	wxSTC_LEX_CONTAINER
+our $VERSION = '0.18';
 
-	wxSTC_EOL_CRLF
-	wxSTC_EOL_CR
-	wxSTC_EOL_LF
-};
-use Wx::STC;
-
-use Padre::Util;
-
-our $VERSION = '0.17';
-
-my $cnt   = 0;
+my $unsaved_number = 0;
 
 our %mode = (
-	WIN  => wxSTC_EOL_CRLF,
-	MAC  => wxSTC_EOL_CR,
-	UNIX => wxSTC_EOL_LF,
+	WIN  => Wx::wxSTC_EOL_CRLF,
+	MAC  => Wx::wxSTC_EOL_CR,
+	UNIX => Wx::wxSTC_EOL_LF,
 );
 
 # see Wx-0.86/ext/stc/cpp/st_constants.cpp for extension
@@ -124,53 +86,52 @@ our %EXT_MIME = (
 	yml   => 'text/yaml',
 	yaml  => 'text/yaml',
 	'4th' => 'text/forth',
-
-	pasm  => 'text/pasm',
-	pir   => 'text/pir',
+	pasm  => 'application/x-pasm',
+	pir   => 'application/x-pir',
 	p6    => 'application/x-perl6',
 );
 
 our %MIME_CLASS = (
 	'application/x-perl'  => 'Padre::Document::Perl',
 	'application/x-perl6' => 'Padre::Document::Perl6',
-	'text/pasm'  => 'Padre::Document::Pasm',
-	'text/pir'   => 'Padre::Document::Pir',
-	'text/ecmascript' => 'Padre::Document::JavaScript',
+	'application/x-pasm'  => 'Padre::Document::Pasm',
+	'application/x-pir'   => 'Padre::Document::Pir',
+	'text/ecmascript'     => 'Padre::Document::JavaScript',
 );
 
 our %MIME_LEXER = (
-	'text/x-adasrc'       => wxSTC_LEX_ADA,
-	'text/asm'            => wxSTC_LEX_ASM,
-	'text/bat'            => wxSTC_LEX_BATCH,
-	'text/x-c++src'       => wxSTC_LEX_CPP,
-	'text/css'            => wxSTC_LEX_CSS,
-	'text/x-patch'        => wxSTC_LEX_DIFF,
-	'text/eiffel'         => wxSTC_LEX_EIFFEL,
-	'text/forth'          => wxSTC_LEX_FORTH,
-	'text/x-fortran'      => wxSTC_LEX_FORTRAN,
-	'text/html'           => wxSTC_LEX_HTML,
-	'text/ecmascript'     => wxSTC_LEX_ESCRIPT,
-	'text/latex'          => wxSTC_LEX_LATEX,
-	'text/lisp'           => wxSTC_LEX_LISP,
-	'text/lua'            => wxSTC_LEX_LUA,
-	'text/x-makefile'     => wxSTC_LEX_MAKEFILE,
-	'text/matlab'         => wxSTC_LEX_MATLAB,
-	'text/x-pascal'       => wxSTC_LEX_PASCAL,
-	'application/x-perl'  => wxSTC_LEX_PERL,
-	'text/x-python'       => wxSTC_LEX_PYTHON,
-	'application/x-php'   => wxSTC_LEX_PHPSCRIPT,
-	'application/x-ruby'  => wxSTC_LEX_RUBY,
-	'text/x-sql'          => wxSTC_LEX_SQL,
-	'text/x-tcl'          => wxSTC_LEX_TCL,
-	'text/vbscript'       => wxSTC_LEX_VBSCRIPT,
-	'text/xml'            => wxSTC_LEX_XML,
-	'text/yaml'           => wxSTC_LEX_YAML,
-	'text/pir'            => wxSTC_LEX_CONTAINER,
-	'text/pasm'           => wxSTC_LEX_CONTAINER,
-	'application/x-perl6' => wxSTC_LEX_CONTAINER,
+	'text/x-adasrc'       => Wx::wxSTC_LEX_ADA,
+	'text/asm'            => Wx::wxSTC_LEX_ASM,
+	'text/bat'            => Wx::wxSTC_LEX_BATCH,
+	'text/x-c++src'       => Wx::wxSTC_LEX_CPP,
+	'text/css'            => Wx::wxSTC_LEX_CSS,
+	'text/x-patch'        => Wx::wxSTC_LEX_DIFF,
+	'text/eiffel'         => Wx::wxSTC_LEX_EIFFEL,
+	'text/forth'          => Wx::wxSTC_LEX_FORTH,
+	'text/x-fortran'      => Wx::wxSTC_LEX_FORTRAN,
+	'text/html'           => Wx::wxSTC_LEX_HTML,
+	'text/ecmascript'     => Wx::wxSTC_LEX_ESCRIPT,
+	'text/latex'          => Wx::wxSTC_LEX_LATEX,
+	'text/lisp'           => Wx::wxSTC_LEX_LISP,
+	'text/lua'            => Wx::wxSTC_LEX_LUA,
+	'text/x-makefile'     => Wx::wxSTC_LEX_MAKEFILE,
+	'text/matlab'         => Wx::wxSTC_LEX_MATLAB,
+	'text/x-pascal'       => Wx::wxSTC_LEX_PASCAL,
+	'application/x-perl'  => Wx::wxSTC_LEX_PERL,
+	'text/x-python'       => Wx::wxSTC_LEX_PYTHON,
+	'application/x-php'   => Wx::wxSTC_LEX_PHPSCRIPT,
+	'application/x-ruby'  => Wx::wxSTC_LEX_RUBY,
+	'text/x-sql'          => Wx::wxSTC_LEX_SQL,
+	'text/x-tcl'          => Wx::wxSTC_LEX_TCL,
+	'text/vbscript'       => Wx::wxSTC_LEX_VBSCRIPT,
+	'text/xml'            => Wx::wxSTC_LEX_XML,
+	'text/yaml'           => Wx::wxSTC_LEX_YAML,
+	'application/x-pir'   => Wx::wxSTC_LEX_CONTAINER,
+	'application/x-pasm'  => Wx::wxSTC_LEX_CONTAINER,
+	'application/x-perl6' => Wx::wxSTC_LEX_CONTAINER,
 );
 
-our $DEFAULT_LEXER = wxSTC_LEX_AUTOMATIC;
+our $DEFAULT_LEXER = Wx::wxSTC_LEX_AUTOMATIC;
 
 
 
@@ -261,7 +222,7 @@ sub setup {
 	if ( $self->{filename} ) {
 		$self->{newline_type} = $self->load_file($self->{filename}, $self->editor);
 	} else {
-		$cnt++;
+		$unsaved_number++;
 		$self->{newline_type} = $self->_get_default_newline_type;
 	}
 }
@@ -271,7 +232,7 @@ sub get_title {
 	if ( $self->{filename} ) {
 		return File::Basename::basename( $self->{filename} );
 	} else {
-		return " Unsaved Document $cnt";
+		return " Unsaved $unsaved_number";
 	}
 }
 
@@ -325,7 +286,7 @@ sub load_file {
 			$convert_to = $newline_type = $mixed;
 		}
 	} else {
-		$convert_to = $self->_auto_convert();
+		$convert_to = $self->_auto_convert;
 		if ($convert_to) {
 			#warn "TODO call converting on $file";
 			$newline_type = $convert_to;
@@ -347,9 +308,8 @@ sub load_file {
 
 sub save_file {
 	my ($self) = @_;
-	my $content      = $self->text_get;
-	my $filename     = $self->filename;
-    #my $newline_type = $self->get_newline_type;
+	my $content  = $self->text_get;
+	my $filename = $self->filename;
         
 	require File::Slurp;
 	eval {
@@ -610,8 +570,8 @@ sub remove_color {
 	my ($self) = @_;
 
 	my $editor = $self->editor;
-	# TODO this is strange, do we reall need to do it with all?
-	for my $i (0..31) {
+	# TODO this is strange, do we really need to do it with all?
+	for my $i ( 0..31 ) {
 		$editor->StartStyling(0, $i);
 		$editor->SetStyling($editor->GetLength, 0);
 	}
@@ -619,6 +579,50 @@ sub remove_color {
 	return;
 }
 
+#
+# $doc->comment_lines_str;
+#
+# this is of course dependant on the language, and thus it's actually
+# done in the subclasses. however, we provide base empty methods in
+# order for padre not to crash if user wants to un/comment lines with
+# a document type that did not define those methods.
+#
+sub comment_lines_str {}
+
+sub stats {
+	my ($self) = @_;
+	
+	my ( $lines, $chars_with_space, $chars_without_space, $words, $is_readonly )
+		= (0) x 5;
+
+	my $editor = $self->editor;
+	my $src = $editor->GetSelectedText;
+	my $code;
+	if ( $src ) {
+		$code = $src;
+
+		my $code2 = $code; # it's ugly, need improvement
+		$code2 =~ s/\r\n/\n/g;
+		$lines = 1; # by default
+		$lines++ while ( $code2 =~ /[\r\n]/g );
+		$chars_with_space = length($code);
+    } else {
+		$code = $self->text_get;
+
+		# I trust editor more
+		$lines = $editor->GetLineCount();
+		$chars_with_space = $editor->GetTextLength();
+		$is_readonly = $editor->GetReadOnly();
+	}
+    
+	$words++ while ( $code =~ /\b\w+\b/g );
+	$chars_without_space++ while ( $code =~ /\S/g );
+
+	my $filename = $self->filename;
+	
+	return ( $lines, $chars_with_space, $chars_without_space, $words, $is_readonly, $filename);
+}
+
 1;
 
 # Copyright 2008 Gabor Szabo.
