@@ -10,7 +10,7 @@ use Padre::Wx;
 use Padre::Wx::Dialog;
 use Wx::Locale qw(:default);
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 my @cbs = qw(case_insensitive use_regex backwards close_on_hit);
 
@@ -101,18 +101,19 @@ sub find_next {
 	my ($class, $main) = @_;
 	
 	my $config = Padre->ide->config;
+	my $term = $config->{search_terms}->[0];
+	
 	# for Quick Find
 	if ( $config->{experimental} ) {
 		# check if is checked
 		if ( $main->{menu}->{experimental_quick_find}->IsChecked ) {
 			my $text = $main->selected_text;
-			if ( $text ) {
+			if ( $text and $text ne $term ) {
 				unshift @{$config->{search_terms}}, $text;
 			}
 		}
 	}
 
-	my $term = $config->{search_terms}->[0];
 	if ( $term ) {
 		$class->search();
 	} else {
